/* filename:    gibssdd.g
** description: Computes numerical approximations to the data density
**              based on MH output
*/

new;
closeall;
library user, pgraph;
cls;
outwidth 128;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g


mhrun = "1";
nburn0 = 21;
nburn1 = 21;
nblock0 = 200;
nblock1 = 200;

/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   
SS    = series_sT;  


/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prt.out";
load path = ^priopath prior[nparat,3] = ^priorfile;

pmean_t  = prior[.,1];
pstdd_t  = prior[.,2];
pshape_t = prior[.,3];
pshape_t = pshape_t.*parat_maskinv[.,mspec];

priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prp.out";
load path = ^priopath prior[nparap,3] = ^priorfile;

pmean_p  = prior[.,1];
pstdd_p  = prior[.,2];
pshape_p = prior[.,3];

/******************************************************** 
** Load Posterior Mode Estimates
** Set phi_star and theta_star for calculation of Marginal DD
*/
lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;

para_t = para[1:nparat];
para_p = para[nparat+1:npara];

load path = ^priopath parat_names;
load path = ^priopath parap_names;
para_names = parat_names ~ parap_names;

"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Subsample        " $subTstr;
"MHrun Index      " $mhrun;
"";
"===================================";
"";
"";
"Parameter   | Estimate ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

"Press Key to Continue";
k = keyw;

/******************************************************** 
** Calculate the likelihood*prior at theta_star, phi_star
*/
cls;
"Computing likelihood and prior at theta_star, phi_star.";
" ";

/* Step 1: Calculate the prior at thetat_star, phi_star
*/
lnprio = priodens(para_t, pmean_t, pstdd_t, pshape_t)
         + priodens(para_p, pmean_p, pstdd_p, pshape_p);

/* Step 2: Calculate likelihood at theta_star, phi_star
*/

/* calculate directly
*/
{lnpYexact,retcode,obserror,obsvar} = evallp(para_t,para_p,mspec,YY); 
lnpYdirect = real(lnpYexact);
"Direct Calculation:   " lnpYdirect;

/* Indirect calculation
*/   
{SSdraw, loglhSSdraw }  = drawpostss(para_t,para_p,mspec,YY);
logpriorSSdraw = priorss(para_p,SSdraw);
{lnpY,retcode,obserror,obsvar} = evallpss(para_t,para_p,mspec,YY,SSdraw); 
lnpYindirect = real(lnpY) + logpriorSSdraw - loglhSSdraw;
"Indirect Calculation: " lnpYindirect;

lnpY = lnpYdirect;
"Press Key to Continue";
k = keyw;


/******************************************************** 
** compute estimate of p(theta_star | YY)
*/
cls;
locate 1,1;
"Computing p(thetastar | YY)";

lpath  = parapath $+ "\\mhrun" $+ mhrun;
lstat  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st";
open  fhstat = ^lstat for read;

statrow = seekr(fhstat,nburn1);
statsim = readr(fhstat,1);

statvec = statsim[.,2:3];
ndraws  = 1;
eofloop = 0;

do until eofloop;

   statsim  = readr( fhstat, nblock1);
   statvec  = statvec | statsim[.,2:3];
   ndraws = ndraws + rows(statsim);

   locate 3,1;
   "Reading MH - Gibbs Output (1) to Compute Data Density";
   "Number of Draws Used:" ndraws;
   eofloop = eof(fhstat);

endo;
closeall fhstat;

statmax = maxc(statvec)';

statvec = cumsumc(exp(statvec-statmax))./seqa(1,1,ndraws);
lnpthetastar = statmax[1]+ln(1E-90+statvec[.,1]) - (statmax[2]+ln(statvec[.,2]));

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_plwidth = 3.5;

begwind;
window(1,1,0);
xy(seqa(1,1,ndraws),lnpthetastar);
endwind;

lnpthetastarhat = lnpthetastar[ndraws,1];

"Estimated Density p(thetastar | YY): " lnpthetastarhat;
"";
"Press Key to Continue";
k = keyw;

/******************************************************** 
** compute estimate of p(phi_star | theta_star, YY)
*/
cls;
locate 1,1;
"Computing p(pstar|thetastar,YY)";

lpath  = parapath $+ "\\mhrun" $+ mhrun $+ "0";
lstat  = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st";
open  fhstat = ^lstat for read;

statrow = seekr(fhstat,nburn0);
statsim = readr(fhstat,1);

statvec = statsim[.,1];
ndraws  = 1;
eofloop = 0;

do until eofloop;

   statsim  = readr( fhstat, nblock0);
   statvec  = statvec | statsim[.,1];
   ndraws = ndraws + rows(statsim);

   locate 3,1;
   "Reading MH - Gibbs Output (1) to Compute Data Density";
   "Number of Draws Used:" ndraws;
   eofloop = eof(fhstat);

endo;

statvec = exp(statvec);

statvec = cumsumc(statvec)./seqa(1,1,ndraws);
lnppstar = ln(statvec[.,1]);

graphset;
fonts("microb");
_ptitlht = 0.4;
_paxht = 0.4;
_pnumht = 0.4;
_pltype = {6 1 2}; 
_plwidth = 3.5;

begwind;
window(1,1,0);
xy(seqa(1,1,ndraws),lnppstar);
endwind;

lnppstarhat = lnppstar[ndraws,1];

"Estimated Density p(phistar | thetastar, YY):"
lnppstarhat;
"";
"Press Key to Continue";
k = keyw;

/******************************************************************
**   Compute data density
*/
cls;

lnpYChib = lnprio + lnpY - ( lnpthetastarhat + lnppstarhat);

"Summary of Calculations";
"";
"p(theta_star, phi_star):      " lnprio;
"p(YY | theta_star, phi_star): " lnpY;
"p(theta_star | YY):           " lnpthetastarhat;
"p(phi_star | theta_star, YY): " lnppstarhat;
" ";
"Data Density:                 " lnpYChib;
end;


#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lplh.src;
#include c:\projects\active\learnpol\gauss2\lpdraw.src;
