/* filename:    lpgibbs0.g
** description: The program generates draws from the posterior
**              distribution of SS and phi via Gibbs sampling
**              
** created:     02/21/02
** modified:   
*/

new;
closeall;
library user, pgraph;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g


/******************************************************** 
**  Configure the MCMC Algorithm
*/
mhrun   = "1" $+ "0";
nblocks = 20;
nsim    = 100;
opath   = parapath $+ "\\mhrun" $+ mhrun;


/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   
SS    = series_sT;  /* We use the imported series SS as starting values */


/******************************************************** 
** Import the information for the prior density
*/
priorfile = lmodel $+ lprior $+ dataselstr $+ subTstr $+ "prp.out";
load path = ^priopath prior[nparap,3] = ^priorfile;

pmean_p  = prior[.,1];
pstdd_p  = prior[.,2];

/******************************************************** 
** Load  theta values and set starting values for para_p
*/
lpara = postpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pm";
open fhpara = ^lpara for read;
para = readr(fhpara,npara);
closeall fhpara;

load path = ^priopath parat_names;
load path = ^priopath parap_names;
para_names = parat_names ~ parap_names;

para_t = para[1:nparat];
para_p = para[nparat+1:npara];

para_pstar = para_p;

"Model            " $lmodel;
"Data set         " $dataselstr;
"Prior            " $lprior;
"Subsample        " $subTstr;
"MHrun Index      " $mhrun;
"";
"===================================";
"";
"";
"Parameter   | Estimates ";
outmat = para_names'~para;
let mask[1,2] = 0 1;
let fmt[2,3] =
   "-*.*s " 10 4
   "*.*lf " 10 4;
d = printfm(outmat,(0 ~ 1),fmt);

"Press key to continue";
k=keyw;
cls;

/******************************************************** 
** Initialize Output files
*/

ostat  = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "st"; 
statname = { PHIDENS }; 

create fhstat =^ostat  with ^statname, 1, 8; 


ophi   = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";
oss    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ps";

create fhphi   = ^ophi   with ^parap_names, nparap, 8;
create fhss    = ^oss    with SS, nobs, 2;

/******************************************************** 
**              Gibbs Algorithm
*/

/* Initialize SS with values read from file
*/
SS_old    = SS;

/* Initialize phi directly
*/
phi_old   = para_p;

iblock = 1;
do until iblock > nblocks;

   phi_sim   = zeros(nsim,2);
   ss_sim    = zeros(nsim,nobs);
   stat_sim  = zeros(nsim,1);
   
   tstart = date;

   j = 1;

   do until j > nsim;

      /* Step 1: draw from SS | theta, phi
      */
      {ss_new, loglhss}  = drawpostss(para_t,phi_old,mspec,YY);

      /* Step 2: draw phi | SS
      */
      phi_new       = drawpostphi(pmean_p~pstdd_p,ss_new); 
      stat_sim[j,1] = postphi(para_pstar,pmean_p~pstdd_p,ss_new);

      /* Store results 
      */
      phi_sim[j,.]   = phi_new';
      SS_sim[j,.]    = SS_new[.,2]';
       
      /* Update
      */
      phi_old   = phi_new;
      SS_old    = SS_new;

      /*
      locate 1,1;
      "Block" iblock "of" nblocks;
      "Simulation step" j "of" nsim;
      "PHI   "; phi_sim[j,.]; 
      */

      j = j+1;
   endo;

   tend = date;

   locate 1,1;
   "Block" iblock "of" nblocks;
   "PHI   "; meanc(phi_sim)';
   "Elapsed Time"; etstr(ethsec(tstart,tend));

    wr = writer(fhstat,stat_sim);
    wr = writer(fhphi,phi_sim);
    wr = writer(fhss,ss_sim);

    iblock = iblock+1;
endo;  

closeall fhphi, fhss, fhstat;
end;

#include c:\projects\active\learnpol\gauss2\lpdraw.src;
