/* filename:    lpprior2.g
** description: The program deletes the draws that lie outside of the parameter space
** created:     12/15/01
*/

new;
closeall;
library pgraph, user;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g

/* Define some Constants
*/
nsim   = 2000;   /* block size */
nobs   = 152;
mhrun  = "0";

/* Open files with Parameter Draws
*/
lpath  = parapath $+ "\\mhrun" $+ mhrun;  

ltheta = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt0";
open fhltheta = ^ltheta for read;

lphi   = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp0";
open fhlphi = ^lphi for read;

lss   = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ps0";
open fhlss = ^lss for read;

/* Initialize Output files
*/
opath = lpath;

otheta = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
create fhotheta = ^otheta with THETA, nparat, 8;

ophi = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";
create fhophi = ^ophi with PHI, nparap, 8;

oss   = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ps";
create fhoss = ^oss with SS, nobs, 8;

odet  = opath  $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "dt";
create fhdet = ^odet with DET, 2, 2;

/* Initialization
*/ 
eofloop = 0;
loopct  = 0;
nread   = 0;
nvalid  = 0;

do until eofloop; 

   /* Read blocks of size nsim
   */
   thetasim = readr( fhltheta, nsim );
   phisim   = readr( fhlphi, nsim );
   sssim    = readr( fhlss, nsim );

   nread   = nread + rows(thetasim);

   /* Eliminate draws that are in the non-existence and indeterminacy region
   */
   invaliddraw = zeros(rows(thetasim),1);
   detsim      = zeros(rows(thetasim),2);

   j = 1;
   do until j > rows(thetasim);

      /* For each element of the block, compute the desired statistic
      */

      {T1,TC,T0,EXPeffect_f,EXPeffect_m,RC} = dsgesolv(thetasim[j,.]',phisim[j,.]');
      
      if (RC[1] == 1) AND (RC[2]==0);
         /* indeterminacy */
         detsim[j,2] = 1;
         
      elseif (RC[1] == 1) AND (RC[2]==1); 
         /* determinacy */
         detsim[j,1] = 1; 
       
      else;
         invaliddraw[j,1] = 1;  

      endif; 

      j = j+1;
  endo;

  thetasim = delif(thetasim, invaliddraw);
  phisim   = delif(phisim, invaliddraw);
  sssim    = delif(sssim, invaliddraw);
  
  detsim   = delif(detsim, invaliddraw);
  
  thetasim = delif(thetasim, detsim[.,2]);
  phisim   = delif(phisim, detsim[.,2]);
  sssim    = delif(sssim, detsim[.,2]);
  
  nvalid  = nvalid + rows(thetasim);

  /* Write Output
  */
  wr = writer(fhdet,detsim);
  wr = writer(fhotheta,thetasim);
  wr = writer(fhophi,phisim);
  wr = writer(fhoss,sssim);
  
  loopct = loopct + 1;

  locate 1,1;
  "Loop            " loopct;
  "Relative Size of Regions [det, indet]"; meanc(detsim)';
  eofloop = eof(fhltheta);
  
endo;

"Fraction of valid draws" nvalid/nread;

closeall fhdet, fhltheta, fhlphi, fhlss, fhotheta, fhophi, fhoss;

#include c:\projects\active\learnpol\gauss2\lpmod.src;
#include c:\projects\active\learnpol\gauss2\lplh.src;
