/*************************************************************************
**  Simulate the DSGE model 
**************************************************************************/

/* filename: lpsim.g
** created:  10/18/01
** Modifications:
**
*/
new;
library pgraph, user;
  _fcmptol = 1E-10;
#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g
#include c:\projects\active\learnpol\gauss2\lpmod.src

cls;
   
/* Load model parameters
*/
load path = ^priopath lppara[nparat,1] = "msim_par.out";
load path = ^priopath lpphi[nparap,1]      = "msim_phi.out";
para      = lppara[.,datasel];
phi       = lpphi[.,datasel];

gam     = para[1];
pilq    = para[2]/4;
pihq    = para[2]/4 + para[3]/4;
pila    = 4*pilq;
piha    = 4*pihq;
rsta    = para[4];
psi1    = para[7];
rhor    = para[10];
sigg    = para[11];
sigz    = para[12];
sigrlq  = para[13]/4;
sigrhq  = para[14]/4; 

/* 
** s0:        state of economy in t0
** x0:        initial value of x
** probs0:    agents beliefs about state of economy in t0
** sT:        sequence of states
** yT:        sequence of observed variables
** xT:        sequence of latent variables:
**            [y,inf,r2,xi_y,xi_infl,g,z]'
*/

/* Markov Transition Matrix
*/
strans = ( phi[1] ~ (1 - phi[2]) )|
         ( (1 - phi[1]) ~ phi[2] );

/* Calculate Steady State Inflation 
** at quarterly frequency
*/
piq  = (1-phi[2])/(2-phi[1]-phi[2])*pilq
     + (1-phi[1])/(2-phi[1]-phi[2])*pihq;


pist = (pilq | pihq) - piq;

/* Solve the DSGE model */

{T1,TC,T0,EXPeffect_f,EXPeffect_m,RC} = dsgesolv(para,phi);

/* Compute Impulse Response functions 
** Composition of xt: [y,inf,r2,xi_y,xi_infl,g,z]'
*/
ZZ      = zeros(3,rows(T1));
ZZ[1,1] = 1; ZZ[1,7] = 1;  ZZ[1,8] = - 1; /* output growth */
ZZ[2,2] = 4;               /* annualized inflation */
ZZ[3,3] = 4;               /* annualized interest rate */

DD      = gam | 4*piq | 4*piq+rsta;

/* Initializations 
*/
yT = zeros(Tsimtotal+Tsimburn,3);
sT = zeros(Tsimtotal+Tsimburn,2);

xt = zeros(cols(T1),1);
s0 = (1 | 0);
probst = ( 0.5 | 0.5);


t = 1;
do until t > Tsimburn + Tsimtotal;

   /* Update the state */
   probtrans = strans*s0;
   if rndu(1,1) < probtrans[1];
      s0[1] = 1;
      s0[2] = 0;
   else;
      s0[1] = 0;
      s0[2] = 1;
   endif;
   sT[t,.] = s0';

   /* calculate realization of epst */
   epst   = rndn(1,1)*sigg |
            rndn(1,1)*sigz |
            (1-rhor)*(1-psi1)*(sT[t,.]*pist) + rndn(1,1)*(sT[t,.]*(sigrlq|sigrhq));

   if mspec == 1;

      /* CASE 1: full information */
      probst = sT[t,.]'; 
      EXPepsr = EXPeffect_f*probst;
       
   elseif mspec == 2;
   
      /* CASE 2: myopic beliefs */
      probst = sT[t,.]'; 
      EXPepsr   = EXPeffect_m*probst;
   
   else;
   
      /* Bayesian Learning */
      probstt1 = strans*probst;
      zetat =  exp( -0.5 * ( ( (1|1)*epst[3] - (1-rhor)*(1-psi1)*pist )^2 ./ (sigrlq^2 | sigrhq^2) ) ) ./ (sigrlq | sigrhq) ; 
      probst = ( probstt1 .* zetat )/sumc( probstt1 .* zetat );
      EXPepsr   = EXPeffect_f*probst;

   endif;
  
   /* Update vector x(t) */
   xt = T1*xt + TC + T0*epst + EXPepsr;
   yT[t,.] = DD' + (ZZ*xt)';

   /* Recover epsrt */
   /*
   if t > 1;
      yhat = (1-rhor)*rsta + (1-rhor)*psi1*yT[t,2] + 
             (1-rhor)*(1-psi1)*(sT[t,1]*4*pilq + sT[t,2]*4*pihq) + rhor*yT[t-1,3]; 
      nu   = (yT[t,3] - yhat)^2/(sT[t,1]*(4*sigrlq)^2 + sT[t,2]*(4*sigrhq)^2);

      epsrt = (epst[3] - (1-rhor)*(1-psi1)*(sT[t,.]*pist))^2 / (sT[t,.]*(sigrlq^2|sigrhq^2));
   
      locate 1,1;
      "nu       " nu;
      "epsrt    " epsrt;
   endif;

   k = keyw;
   */      

   t = t+1;

endo;

yT = yT[Tsimburn+1:Tsimtotal+Tsimburn,.];
sT = sT[Tsimburn+1:Tsimtotal+Tsimburn,.];

"Data Simulated Successfully";
"Parameter Choice:    " datasel;
"Model Specification: " mspec;
"Parameters (theta):  "; para;
"Parameters (phi):    "; phi';
"Mean of ST:          " meanc(sT)';
"Mean of YT:          " meanc(yT)';
"Mean of YT in R1:    " sumc(yt.*sT[.,1])'/sumc(sT[.,1]);
"Mean of YT in R2:    " sumc(yt.*sT[.,2])'/sumc(sT[.,2]);
"StD of YT:           " stdc(yT)';

eofsunsim:

/********************************************************
** Save simulated data
*/

osimdata         = datapath $+ "\\" $+ lmodel $+ "yT" $+ dataselstr;
create fhsimdata = ^osimdata with SERIES, 3, 8;
wr               = writer(fhsimdata,yT);
closeall fhsimdata;   

osimdata         = datapath $+ "\\" $+ lmodel $+ "sT" $+ dataselstr;
create fhsimdata = ^osimdata with SERIES, 2, 8;
wr               = writer(fhsimdata,sT);
closeall fhsimdata;   

"Data saved";
