/* filename:    ss_draw.g
** description: The program generates draws from the posterior
**              of SS conditional on theta and phi
**              
** created:     02/21/02
** modified:   
*/

new;
closeall;
library user, pgraph;
cls;

/* Global defaults 
*/
  __output = 1;
  _fcmptol = 1E-10;

#include c:\projects\active\learnpol\gauss2\pathspec.g
#include c:\projects\active\learnpol\gauss2\lpspec.g

nblock  = 1000;
mhrun   = "2";
opath   = parapath $+ "\\mhrun" $+ mhrun;

/******************************************************** 
** Import data
** series (nobs,3)
*/
#include c:\projects\active\learnpol\gauss2\loaddata.g

nobs  = rows(series_yT);  /* number of observations */
YY    = series_yT;   


/******************************************************** 
** Initialize Output files
*/
oss    = opath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "ps";
create fhss = ^oss    with SS, nobs, 2;

/******************************************************** 
** Open files with theta and phi draws
*/
lpath  = parapath $+ "\\mhrun" $+ mhrun;
ltheta = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pt";
lphi   = lpath $+ "\\" $+ lmodel $+ lprior $+ dataselstr $+ subTstr $+ "pp";

open fhphi   = ^lphi for read;
open fhtheta = ^ltheta for read;

/******************************************************** 
** Read files with theta and phi draws
*/

blockind = 1;
eofloop  = 0;
nuse     = 4;    /* use every nuse observation */

do until eofloop; 

   tstart = date;

   theta_sim = readr( fhtheta, nblock );
   phi_sim   = readr( fhphi, nblock) ;
   nread     = rows(phi_sim);   

   indseq = seqa(1,1,nread);
   indseq = indseq % nuse;
   indseq = indseq .NE 0;
 
   theta_sim = delif(theta_sim,indseq);
   phi_sim   = delif(phi_sim,indseq);

   ssT_sim   = zeros( rows(phi_sim),nobs );

   i = 1;
   do until i > rows(phi_sim);
   
      { SSdraw,loglh } = drawpostss(theta_sim[i,.]',phi_sim[i,.]',mspec,YY);
      ssT_sim[i,.] = SSdraw[.,2]';
      
      i = i+1;
      
   endo;
  
   wr = writer(fhss, ssT_sim);
   eofloop = eof(fhtheta);
   tend = date;

   locate 1,1;
   "Block Number:   " blockind;
   "Draws Processed:" rows(phi_sim);
   "Elapsed Time:   " etstr(ethsec(tstart,tend));
   blockind = blockind + 1;

endo;

closeall fhss, fhtheta, fhphi;

#include c:\projects\active\learnpol\gauss2\lpdraw.src;
#include c:\projects\active\learnpol\gauss2\lpmod.src;
